package com.abacus.improved;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.JFrame;

import com.abacus.legacy.EditNodeDialog;
import com.abacus.legacy.Node;

/**
 * Main editor panel for program. Since Its parent class utilizes Mouse Events to implement the zoom function,
 * an inner class has been created to allow node manipulation within the editor.
 * @author saiumesh
 *
 */
public class NodeEditorPanel extends AbstractZoomablePanel{
	private MouseAction mouseAction;
	private boolean canEdit = true;
	ArrayList<Node> nodes;
	protected Image previewImage = null;
	private Integer selectedIndex = -1, selectedState = -1;
	EditNodeDialog editDialog;
	Point mousePoint, lastMousePoint, startPoint;
	Object selection = null;

	public NodeEditorPanel(MachinePanel parentPanel) {
		this.nodes = parentPanel.getNodeList();
		this.mouseAction = new MouseAction();
		setBackground(Color.white);
		this.setBorder(BorderFactory.createLineBorder(Color.black));
		this.editDialog = new EditNodeDialog((JFrame) parentPanel.getParent());
		BackgroundTask task = new BackgroundTask(parentPanel);
		task.start();
		
		addMouseListener(mouseAction);
		addMouseMotionListener(mouseAction);
		addMouseWheelListener(mouseAction);
	}

	@Override
	protected void draw(Graphics2D g) {
		//update node list in parent panel
		if(canEdit)
			((MachinePanel) this.getParent()).setNodeList(nodes);
		
		for (int x = 0; x < nodes.size(); ++x) {
			Node n = (Node)nodes.get(x);
			if (x == selectedIndex && !canEdit)
				n.simSelect(selectedState);
			n.draw(g);
		}
		
		if (nodes.size() > 0) {
			Node n = (Node)nodes.get(0);
			
			n.drawInitState(g);
		}
		
		drawMouseImage(g);
	}
	
	private void drawMouseImage(Graphics2D g)
	{
		if (previewImage != null)
		{
			g.drawImage(previewImage,mousePoint.x - previewImage.getWidth(null) / 2,
					mousePoint.y - previewImage.getHeight(null) / 2,null);
		}
	}
	
	//lock and unlock the panel to prevent edits as machine is running
	public void lockPanel() {
		canEdit = false;
	}
	
	public void unlockPanel() {
		canEdit = true;
	}
	
	public void clearSelection() {
		for (int i = 0; i < this.nodes.size(); i++) {
			nodes.get(i).clearSelection();
		}
		repaint();
	}
	
	//TODO: delete
	class BackgroundTask extends Thread {
		private MachinePanel pnl;
		public BackgroundTask(MachinePanel parentPanel) {
			this.pnl = parentPanel;
		}

		public void run() {
			while(true){
				zoomState = -1 * pnl.getState();
				try {
					Thread.sleep(10);
				} catch (Exception e) { }
			}
		}
		
	}
	
	/**
	 * Allows node manipulation within the panel.
	 * @author saiumesh
	 *
	 */
	private class MouseAction implements MouseListener, MouseMotionListener, MouseWheelListener {
		private Integer lastClickedObject = 0;
		private Node lastClickedNode = null;
		
		@Override
		public void mouseWheelMoved(MouseWheelEvent e) {
			mousePoint = toRealCoords(lastMousePoint);
			repaint();
		}

		@Override
		public void mouseDragged(MouseEvent e) {
			
			if (canEdit && startPoint != null && selection != null)
			{
				Point p = toRealCoords(e.getPoint());
				
				int dx = p.x - startPoint.x;
				int dy = p.y - startPoint.y;
				
				Point oldLoc = (selection instanceof Node) ? ((Node)selection).getLocation() 
						: (new Point(0, 0));
				
				if (selection instanceof Node)
				{
					((Node)selection).setLocation(new Point(oldLoc.x + dx,oldLoc.y + dy));
				}
				
				repaint();
				startPoint = p;
			}
		}

		@Override
		public void mouseMoved(MouseEvent e) {
			if(canEdit && ((MachinePanel) getParent()).isNodeTabSelected()) {
				int state = ((MachinePanel) getParent()).getState();
				lastMousePoint = e.getPoint();
				mousePoint = toRealCoords(e.getPoint());
				
				if(state == ButtonPanel.ADDSTATE){
					previewImage = ButtonPanel.transparentAddImage;
				} else if(state == ButtonPanel.SUBSTATE) {
					previewImage = ButtonPanel.transparentSubImage;
				} else {
					previewImage = null;
				}
			}
			
			repaint();
		}

		@Override
		public void mouseClicked(MouseEvent e) {}

		@Override
		public void mouseEntered(MouseEvent arg0) { }

		@Override
		public void mouseExited(MouseEvent arg0) {
			previewImage = null;
			repaint();
		}
		
		private void modifyNodeState(Node node, Point loc, boolean isAddingNode) {
			int index = nodes.indexOf(node);
			Point p = ((MachinePanel) getParent()).getLocation();
			p.x += getX() + loc.x;
			p.y += getY() + loc.y;
			
			int rv = editDialog.modifyNode(node ,index == 0, p);
			
			if (index != 0 && rv == EditNodeDialog.MOD_MAKEINITIAL) {
				Node temp = nodes.get(index);
				nodes.set(index,nodes.get(0));
				nodes.set(0,temp);
			} else if (rv < 0 && isAddingNode) {
				deleteNode(node, index);
			}
			
		}
		
		private void deleteNode(Node node, int index) {
			for (int x = nodes.size()-1; x >= 0; --x) {
				Node n = (Node)nodes.get(x);
				
				if (n.getOut() == node)
					n.setOut(null);
				else if (n.getOutEmpty() == node)
					n.setOutEmpty(null);
			}
			
			nodes.remove(index);
		}

		@Override
		public void mousePressed(MouseEvent e) {
			Point point = e.getPoint();
			if (!isZoomAction(point) && canEdit && e.getButton() == MouseEvent.BUTTON1 &&
					((MachinePanel) getParent()).isNodeTabSelected()) {
				int state = ((MachinePanel) getParent()).getState();
				lastMousePoint = e.getPoint();
				lastClickedObject = 1;
				mousePoint = toRealCoords(point);
				//when state < 0 => user is not using the Node tab and therefore the Zoom tab...
				//Zoom tab controls to be pushed to the superclass to replace the corner button.
				if(state < 0) {
					return;
				}
				//TODO: change to only add state, mod state and delete state
				if (state == ButtonPanel.ADDSTATE) {
					Node n = new Node();
					n.setPlus(true);
					n.setLocation(new Point(mousePoint.x + 8,mousePoint.y - 14));
					nodes.add(n);
					//modifyNodeState(n, e.getPoint(), true);
					
				} else if (state == ButtonPanel.SUBSTATE) {
					Node n = new Node();
					n.setLocation(new Point(mousePoint.x,mousePoint.y-13));
					nodes.add(n);
				} else {
					if(selection != null) {
						Node applyTo = null;
						int index = -1;
						//figure out where in the list the selected node exists.
						for (int x = nodes.size()-1; x >= 0; --x) {
							Node n = (Node)nodes.get(x);
							
							if (n.isInNode(mousePoint) == true) {
									
								index = x;
								applyTo = n;
								break;
							}
						}
						
						if(applyTo != null && selection != null) {
						
							Node selected = (Node) selection;
							if (selected == lastClickedNode)
								lastClickedObject++;
							if (state == ButtonPanel.MODSTATE) {
								int sel = selected.getSelected();
								
								if (sel == Node.SELECTED_OUT) {
									selected.setOut(applyTo);
								} else if (sel == Node.SELECTED_OUTEMPTY) {
									if (selected == applyTo)
										selected.setOutEmpty(null);
									else
										selected.setOutEmpty(applyTo);
								} else if (sel == Node.SELECTED_NODE && selected == applyTo &&
										lastClickedObject == 2) {								
									Point p = getParent().getLocation();
									p.x += getX() + e.getPoint().x;
									p.y += getY() + e.getPoint().y;
									//this brings up an editor panel...what's the point with the new delete button?
									modifyNodeState(selected, e.getPoint(), false);
									lastClickedNode = null;
								}
								lastClickedObject = 0;
							} else if (state == ButtonPanel.DELSTATE) {
								deleteNode(selected, index);
							}
							((Node) selection).clearSelection();
							selection = null;
						}
					} else if (selection == null) {
						for (int x = 0; x < nodes.size(); ++x) {
							Node n = (Node)nodes.get(x);
							
							if (selection == null && n.select(mousePoint) == true) {
								selection = n;
								lastClickedNode = n;
								if (n.getSelected() == Node.SELECTED_NODE) {
									startPoint = new Point(mousePoint.x, mousePoint.y);
								}
								
								break;
							}
							else if (selection != null) {
								n.clearSelection();
								lastClickedNode = null;
							}
						}
					}
				}
				repaint();
			}
		}

		@Override
		public void mouseReleased(MouseEvent arg0) {
			startPoint = null;
		}
		
	}

	public void setNodeList(ArrayList<Node> nodeList) {
		this.nodes = nodeList;
	}

	public synchronized void updateNodeList(Node updatedNode) {
		for (int i = 0; i < this.nodes.size(); i++) {
			if(this.nodes.get(i) == updatedNode) {
				//this.nodes.get(i).simSelect(updatedNode.getSelected());
				this.selectedIndex = i;
				this.selectedState = updatedNode.getSelected();
			} else {
				nodes.get(i).clearSelection();
			}
		}
		repaint();
	}

}
